/* Copyright (C) 1994 Queen Mary and Westfield College, University of London

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   A copy of the GNU General Public License may be found in
   doc/COPYING in the source tree of this programm.

   This file was written by: Franz Gerschwiler.(goblin@dcs.qmw.ac.uk)

 ****

   modified by: p.hlstede	(peter@calvados.chemie.uni-bremen.de)

   - the code was reorganized and ansified
   - the cursor handling was made a bit easier
   - the display of help messages was moved from the window title
     to a status line

							06.09.1997
 */

#include <string.h>
#include <stdlib.h>

#include <X11/cursorfont.h>
#include <Xm/Xm.h>
#include <Xm/MessageB.h>
#include <Xm/SelectioB.h>
#include <Xm/FileSB.h>
#include <Xm/Text.h>
#include <Xm/PushB.h>

#include "sys_consts.h"
#include "globals.h"
#include "callbacks.h"
#include "utils.h"
#include "dstring.h"

#include "gloe_icon.h"

/***  GLOBAL VARIABLES  ***/
Pixmap  gloe_icon;


/***  VARIABLE DECLARATIONS  ***/
static int   button_pressed;


/***  PROCEDURE DECLARATIONS  ***/
static void  gui_errorCB(Widget w, XtPointer client_data, XtPointer call_data);
static void  gui_queryCB(Widget w, XtPointer client_data, XtPointer call_data);



void  gui_info(char *message)
{
   static Widget  info_DI;
   XmString       msg;

   if (info_DI == (Widget)0)
   {
      info_DI = XmCreateInformationDialog(toplevel, "InfoBox", NULL, 0);

      /* now turn off unwanted buttons */
      XtUnmanageChild(XmMessageBoxGetChild(info_DI, XmDIALOG_HELP_BUTTON));
      XtUnmanageChild(XmMessageBoxGetChild(info_DI, XmDIALOG_CANCEL_BUTTON));
   }

   msg = XmStringCreateLtoR(message, CHARSET);
   XtVaSetValues(info_DI, XmNmessageString, msg, NULL);
   XmStringFree(msg);

   XBell(XtDisplay(info_DI), BELL_VOL);
   XtManageChild(info_DI);
}


int  gui_error(char *message)
{
   static Widget  error_DI;
   XEvent         app_event;
   XmString       msg;

   if (error_DI == (Widget)0)
   {
      error_DI = XmCreateErrorDialog(toplevel, "ErrorBox", NULL, 0);

      XtAddCallback(error_DI, XmNokCallback, gui_errorCB, (XtPointer)YES);
      XtAddCallback(error_DI, XmNhelpCallback, gui_errorCB, (XtPointer)HELP);

      /* now turn off unwanted buttons */
      XtUnmanageChild(XmMessageBoxGetChild(error_DI, XmDIALOG_CANCEL_BUTTON));
   }

   msg = XmStringCreateLtoR(message, CHARSET);
   XtVaSetValues(error_DI, XmNmessageString, msg, NULL);
   XmStringFree(msg);

   XBell(XtDisplay(error_DI), BELL_VOL);
   XtManageChild(error_DI);

   button_pressed = NOTHING;
   while (button_pressed == NOTHING)
   {
      XtAppNextEvent(app_context, &app_event);
      XtDispatchEvent(&app_event);
   }
   XtUnmanageChild(error_DI);
   return button_pressed;
}

static void  gui_errorCB(Widget w, XtPointer client_data, XtPointer call_data)
{
   button_pressed = (int)client_data;
}


/* NOT USED ANYMODE !!! */
#if 0
char  *gui_prompt(char *message, char *value)
{
   static Widget  prompt_DI;
   XEvent         app_event;
   XmString       msg, val;

   if (prompt_DI == (Widget)0)
   {
      prompt_DI = XmCreatePromptDialog(toplevel, "Excuse_Me", NULL, 0);

      XtAddCallback(prompt_DI, XmNokCallback, gui_promptCB, (XtPointer)YES);
      XtAddCallback(prompt_DI, XmNcancelCallback, gui_promptCB, (XtPointer)NO);

      XtUnmanageChild(XmSelectionBoxGetChild(prompt_DI, XmDIALOG_HELP_BUTTON));
   }

   msg = XmStringCreateLtoR(message, CHARSET);
   val = XmStringCreateLtoR(value, CHARSET);
   XtVaSetValues(prompt_DI,
		XmNselectionLabelString, msg,
		XmNtextString, val,
		NULL);
   XmStringFree(msg);
   XmStringFree(val);

   XBell(XtDisplay(prompt_DI), BELL_VOL);
   XtManageChild(prompt_DI);

   button_pressed = NOTHING;
   while (button_pressed == NOTHING)
   {
      XtAppNextEvent(app_context, &app_event);
      XtDispatchEvent(&app_event);
   }
   if (button_pressed == YES)
      return(prompt_str);
   else
      return(NULL);
}

static void  gui_promptCB(Widget w, XtPointer client_data, XtPointer call_data)
{
   XmSelectionBoxCallbackStruct
		*cb = (XmSelectionBoxCallbackStruct*)call_data;
   button_pressed = (int)client_data;
   XtFree(prompt_str);
   XmStringGetLtoR(cb->value, CHARSET, &prompt_str);
}
#endif

char  gui_query(char *message)
{
   static Widget  query_DI;
   XEvent         app_event;
   XmString       msg;

   if (query_DI == (Widget) 0)
   {
      Arg      args[5];
      int      n = 0;
      Widget   save_button;

      XtSetArg(args[n], XmNdialogStyle, XmDIALOG_FULL_APPLICATION_MODAL);
      n++;
      query_DI = XmCreateQuestionDialog(toplevel, "queryDI", args, n);

      save_button = XtVaCreateManagedWidget("save", xmPushButtonWidgetClass, query_DI, NULL);
      XtAddCallback(save_button, XmNactivateCallback, gui_queryCB, (XtPointer)SAVE_THEN_DO);

      XtAddCallback(query_DI, XmNokCallback, gui_queryCB, (XtPointer)YES);
      XtAddCallback(query_DI, XmNcancelCallback, gui_queryCB, (XtPointer)NO);

      XtUnmanageChild(XmMessageBoxGetChild(query_DI, XmDIALOG_HELP_BUTTON));
   }

   msg = XmStringCreateLtoR(message, CHARSET);
   XtVaSetValues(query_DI, XmNmessageString, msg, NULL);
   XmStringFree(msg);

   XBell(XtDisplay(query_DI), BELL_VOL);
   XtManageChild(query_DI);

   button_pressed = NOTHING;
   while (button_pressed == NOTHING)
   {
      XtAppNextEvent(app_context, &app_event);
      XtDispatchEvent(&app_event);
   }
   return(button_pressed);
}

static void  gui_queryCB(Widget w, XtPointer client_data, XtPointer call_data)
{
   button_pressed = (int)client_data;
}



void  set_cursor(Widget w, Cursor c)
{
   XDefineCursor(XtDisplay(w), XtWindow(w), c);
   XFlush(XtDisplay(w));
}


void  unset_cursor(Widget w)
{
   XUndefineCursor(XtDisplay(w), XtWindow(w));
   XFlush(XtDisplay(toplevel));
}


void  gui_createIcon(Widget parent)
{
   gloe_icon = XCreateBitmapFromData(XtDisplay(parent),
			RootWindowOfScreen(XtScreen(parent)),
			gloe_icon_bits, gloe_icon_height, gloe_icon_width);
   XtVaSetValues(parent,
		XmNiconPixmap, gloe_icon,
		XmNiconName, "Gloe",
		NULL);
}


/* Change the name of the icon */
void  set_icon_name(char *message)
{
   XtVaSetValues(toplevel, XmNiconName, message, NULL);
}


/* Never used... */
#if 0
/* find the line number according to the position in the text */
int  lineNumber(int position)
{
   char    *text;
   int      linecount;
   int      charcount;

   /* Initialize Variables */
   linecount = 1;
   charcount = -1;

   /* Get data from jump_to text and text canvas */
   text = XmTextGetString(text_canvas);

   /* Nothing sophisticated, each '\n' represents a new line */
   while (charcount < position - 1)
   {
      charcount++;
      if (text[charcount] == '\n')
	 linecount++;
   }
   if (text != NULL)
      XtFree(text);
   return linecount;
}
#endif

/* Change the title message */
void  set_title_string(void)
{
   DString  titleMessage;
   char    *name;

   name = strrchr(FileName, '/');
   if (name) name++;
   else      name = FileName;

   DStringInit(&titleMessage);

   /* First put the progeam nam in at the front */
   DStringCat(&titleMessage, "Gloe - <");

   /* Now put the file name in */
   DStringCat(&titleMessage, name);
   if (changes_counter)
      DStringCat(&titleMessage, "*");
   DStringCat(&titleMessage, ">");

   XtVaSetValues(toplevel, XmNtitle, titleMessage.str, NULL);

   /* Make sure the users sees the message straight away */
   XFlush(XtDisplay(toplevel));

   DStringFree(&titleMessage);
}

/* Change the help message in the status line */
void  set_status_message(char *msg)
{
   XmString str = XmStringCreateLtoR(msg, CHARSET);
   XtVaSetValues(statusBar, XmNlabelString, str, NULL);
   XmStringFree(str);
}
